﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using WpfClient.Annotations;

namespace WpfClient.Model
{
    public class AuthorsFormModel : INotifyPropertyChanged {
        private AuthorModel selectedAuthor;
        private IList<AuthorModel> authors;

        public IList<AuthorModel> Authors {
            get { return authors; }
            set {
                if (Equals(value, authors)) return;
                authors = value;
                OnPropertyChanged();
            }
        }

        public AuthorModel SelectedAuthor {
            get { return selectedAuthor; }
            set {
                if (Equals(value, selectedAuthor)) return;
                selectedAuthor = value;
                OnPropertyChanged();
                OnPropertyChanged("CanDelete");
            }
        }

        public bool CanDelete {
            get { return SelectedAuthor != null && SelectedAuthor.Id.HasValue; }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null) {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    public class AuthorModel : INotifyPropertyChanged {
        private int? id;
        private string name;
        private DateTime? birthDate;

        public int? Id {
            get { return id; }
            set {
                if (value == id) return;
                id = value;
                OnPropertyChanged();
            }
        }

        public string Name {
            get { return name; }
            set {
                if (value == name) return;
                name = value;
                OnPropertyChanged();
            }
        }

        public DateTime? BirthDate {
            get { return birthDate; }
            set {
                if (value.Equals(birthDate)) return;
                birthDate = value;
                OnPropertyChanged();
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null) {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
